@@ -2,14 +2,19 @@
 
 import logging
 from ctypes import HRESULT, POINTER, byref, c_ulong, c_void_p
-from typing import TYPE_CHECKING, ClassVar, List, Optional, Type, TypeVar
+from typing import TYPE_CHECKING, Any, ClassVar, List, Optional, Type, TypeVar
 
-from comtypes import GUID, _CoUninitialize, com_interface_registry
+import comtypes
+from comtypes import GUID, _CoUninitialize
 from comtypes._memberspec import STDMETHOD, ComMemberGenerator, DispMemberGenerator
 from comtypes._post_coinit import _cominterface_meta_patcher as _meta_patch
 from comtypes._post_coinit.instancemethod import instancemethod
 
 if TYPE_CHECKING:
+    from typing import Literal
+    from typing import Union as _UnionT
+
+    from comtypes import hints  # type: ignore
     from comtypes._memberspec import _ComMemberSpec, _DispMemberSpec
 
 logger = logging.getLogger(__name__)
@@ -18,7 +23,7 @@ logger = logging.getLogger(__name__)
 def _shutdown(
     func=_CoUninitialize,
     _debug=logger.debug,
-):
+) -> None:
     # Sometimes, CoUninitialize, running at Python shutdown,
     # raises an exception.  We suppress this when __debug__ is
     # False.
@@ -28,7 +33,7 @@ def _shutdown(
     else:
         try:
             func()
-        except WindowsError:
+        except OSError:
             pass
     # Set the flag which means that calling obj.Release() is no longer
     # needed.
@@ -131,7 +136,7 @@ class _cominterface_meta(type):
 
         return self
 
-    def __setattr__(self, name, value):
+    def __setattr__(self, name: str, value: Any) -> None:
         if name == "_methods_":
             # XXX I'm no longer sure why the code generator generates
             # "_methods_ = []" in the interface definition, and later
@@ -145,11 +150,11 @@ class _cominterface_meta(type):
             self._make_specials()
         type.__setattr__(self, name, value)
 
-    def _make_specials(self):
+    def _make_specials(self) -> None:
         # This call installs methods that forward the Python protocols
         # to COM protocols.
 
-        def has_name(name):
+        def has_name(name: str) -> bool:
             # Determine whether a property or method named 'name'
             # exists
             if self._case_insensitive_:
@@ -164,7 +169,7 @@ class _cominterface_meta(type):
         if has_name("_NewEnum"):
             _meta_patch.iterator(self)
 
-    def _make_case_insensitive(self):
+    def _make_case_insensitive(self) -> None:
         # The __map_case__ dictionary maps lower case names to the
         # names in the original spelling to enable case insensitive
         # method and attribute access.
@@ -197,7 +202,7 @@ class _cominterface_meta(type):
             if self._case_insensitive_:
                 self.__map_case__[name.lower()] = name
 
-    def __get_baseinterface_methodcount(self):
+    def __get_baseinterface_methodcount(self) -> int:
         "Return the number of com methods in the base interfaces"
         result = 0
         for itf in self.mro()[1:-1]:
@@ -216,7 +221,7 @@ class _cominterface_meta(type):
         except KeyError:
             raise AttributeError("this class must define an _iid_")
         else:
-            com_interface_registry[str(iid)] = self
+            comtypes.com_interface_registry[str(iid)] = self  # type: ignore
         # create members
         vtbl_offset = self.__get_baseinterface_methodcount()
         member_gen = ComMemberGenerator(self.__name__, vtbl_offset, self._iid_)
@@ -261,7 +266,10 @@ class _compointer_meta(type(c_void_p), _cominterface_meta):
 class _compointer_base(c_void_p, metaclass=_compointer_meta):
     "base class for COM interface pointer classes"
 
-    def __del__(self, _debug=logger.debug):
+    if TYPE_CHECKING:
+        __com_interface__: ClassVar[Type["IUnknown"]]
+
+    def __del__(self, _debug=logger.debug) -> None:
         "Release the COM refcount we own."
         if self:
             # comtypes calls CoUninitialize() when the atexit handlers
@@ -275,40 +283,39 @@ class _compointer_base(c_void_p, metaclass=_compointer_meta):
                 _debug("Release %s", self)
                 self.Release()  # type: ignore
 
-    def __eq__(self, other):
+    def __eq__(self, other) -> bool:
         if not isinstance(other, _compointer_base):
             return False
         # get the value property of the c_void_p baseclass, this is the pointer value
-        return (
-            super(_compointer_base, self).value == super(_compointer_base, other).value
-        )
+        return super().value == super(_compointer_base, other).value
 
-    def __hash__(self):
+    def __hash__(self) -> int:
         """Return the hash value of the pointer."""
         # hash the pointer values
-        return hash(super(_compointer_base, self).value)
+        return hash(super().value)
 
     # redefine the .value property; return the object itself.
-    def __get_value(self):
+    def __get_value(self) -> "hints.Self":
         return self
 
     value = property(__get_value, doc="""Return self.""")
 
-    def __repr__(self):
-        ptr = super(_compointer_base, self).value
+    def __repr__(self) -> str:
+        ptr = super().value
         return f"<{self.__class__.__name__} ptr=0x{ptr or 0:x} at {id(self):x}>"
 
     # This fixes the problem when there are multiple python interface types
     # wrapping the same COM interface.  This could happen because some interfaces
     # are contained in multiple typelibs.
     #
-    # It also allows to pass a CoClass instance to an api
-    # expecting a COM interface.
+    # It also allows to pass a COMObject instance to an api expecting a COM interface.
     @classmethod
-    def from_param(cls, value):
+    def from_param(
+        cls, value: "_UnionT[None, Literal[0], hints.Self, IUnknown, hints.COMObject]"
+    ) -> Any:
         """Convert 'value' into a COM pointer to the interface.
 
-        This method accepts a COM pointer, or a CoClass instance
+        This method accepts a COM pointer, or a COMObject/CoClass instance
         which is QueryInterface()d."""
         if value is None:
             return None
@@ -322,9 +329,9 @@ class _compointer_base(c_void_p, metaclass=_compointer_meta):
         # Do we need more checks here?
         if cls._iid_ == getattr(value, "_iid_", None):
             return value
-        # Accept an CoClass instance which exposes the interface required.
+        # Accept an COMObject instance which exposes the interface required.
         try:
-            table = value._com_pointers_
+            table = value._com_pointers_  # type: ignore
         except AttributeError:
             pass
         else:
